<?php
/* Post Office Stack Subscription Manager
 * © Joe Workman 2011 
 */
class PostOfficeAPI {

	private $db_conn;
	private $data_dir;
	private $export_auth;
	private $log_to_console;
   
	public function __construct($db_server, $db_username, $db_password, $db_name, $export_auth, $data_dir = ".", $log_to_console = true) {
		$this->data_dir = $data_dir;
		$this->export_auth = $export_auth;
		$this->log_to_console = $log_to_console;
		$this->db_conn = $this->database_connect($db_server, $db_username, $db_password, $db_name);

		// htaccess file to ensure that the exported CSV files are not accessible via the browser and will be downloaded.
		$htaccess = $this->data_dir.'/.htaccess';
		if (!file_exists($htaccess)) {
			$contents = "Options -Indexes\nAddType application/octet-stream .csv\n";
			file_put_contents($htaccess, $contents);
		}
	}
	public function __destruct() {
		// Close the Database if it was connected to
		mysql_close($this->db_conn);
	}

	//---------------------------------------
	// Private Database Functions
	//---------------------------------------   	
	private function query_database() {	
		// Process all queries that are passed
		$queries = func_get_args();
        foreach ($queries as $query) {
			$result = mysql_query($query, $this->db_conn);
			if (!$result) {
				$this->log_message("Problem running query: $query: (".mysql_errno($this->db_conn).'):'.mysql_error($this->db_conn));
				return false;
			}
	    }
		return true;
	}
	private function create_database_tables($db_conn) {	
		// Create the Database Tables unless the created file exists already
		$create_file = $this->data_dir.'/postoffce110.created';
		$legacy_file = $this->data_dir.'/postoffce.created';
		$queries = array();
		
        if (!file_exists($create_file) and file_exists($legacy_file)) {
		    array_push($queries, "ALTER TABLE active_mailing ADD customone varchar(50)", "ALTER TABLE active_mailing ADD customtwo varchar(50)");
		    array_push($queries, "ALTER TABLE ignore_mailing ADD customone varchar(50)", "ALTER TABLE ignore_mailing ADD customtwo varchar(50)");
        }        
        elseif (!file_exists($create_file)) {
		    array_push($queries, "CREATE TABLE IF NOT EXISTS active_mailing (last_name varchar(50) DEFAULT NULL, first_name varchar(50) DEFAULT NULL, email varchar(50) NOT NULL, customone varchar(50) DEFAULT NULL, customtwo varchar(50) DEFAULT NULL, PRIMARY KEY (email))");
		    array_push($queries, "CREATE TABLE IF NOT EXISTS ignore_mailing (last_name varchar(50) DEFAULT NULL, first_name varchar(50) DEFAULT NULL, email varchar(50) NOT NULL, customone varchar(50) DEFAULT NULL, customtwo varchar(50) DEFAULT NULL, PRIMARY KEY (email))");
        }

        foreach ($queries as $query) {
        	$result = mysql_query($query, $db_conn);
        	if (!$result) {
        		$this->log_message("Problem running query: $query: (".mysql_errno($db_conn).'):'.mysql_error($db_conn));
        		return false;
        	}
        }        
	    $fh = fopen($create_file, 'w');
	    fclose($fh);
		return;
	}
	private function database_connect($db_server, $db_username, $db_password, $db_name) {
		// Connect to the database
		$db_conn = mysql_connect($db_server, $db_username, $db_password) or $this->log_message('Could not connect to database server: '.mysql_error());
		$db_name = mysql_select_db($db_name, $db_conn) or $this->log_message('Could not connect to database name: '.mysql_error());

		if ($db_conn && $db_name) {
			$this->create_database_tables($db_conn);
			return $db_conn;
		}
		return;
	}
	private function cleanup_injections($value) { 
		// Clean up to prevent injection attacks
		// Strip slashes if put in already
		if (get_magic_quotes_gpc()) { $value = stripslashes($value); }	
		// Replace html characters - brackets, quotes & Strip %
		$value = str_replace("%", "", htmlspecialchars($value));
		// Quote if not an integer
		return is_numeric($value) ? $value : "'". mysql_real_escape_string($value, $this->db_conn) ."'";
	}

	//---------------------------------------
	// Error Logging
	//---------------------------------------   	
	// set_error_handler("custom_error");
	private function custom_error($errno, $string) {
		return $this->log_message("Error Handler: [$errno] $string");
	}
	private function log_message($string) {
		if ($this->log_to_console) {
			echo '<script type="text/javascript">';
			echo "console.log(\"Post Office: $string\");";
			echo '</script>';
		}
		return;
	}

	//---------------------------------------
	// Public Subscribe Functions
	//---------------------------------------   	
	public function add_subscriber($email, $first_name="", $last_name="", $custom="", $customtwo="") {
		$email 		= $this->cleanup_injections($email);
		$first_name = $this->cleanup_injections($first_name);
		$last_name  = $this->cleanup_injections($last_name);
        $custom     = $this->cleanup_injections($custom);
        $customtwo  = $this->cleanup_injections($customtwo);

		$query = "INSERT IGNORE INTO active_mailing (first_name, last_name, email, customone, customtwo) VALUES($first_name, $last_name, $email, $custom, $customtwo)";
		$query2 = "DELETE FROM ignore_mailing WHERE email = $email";
		return $this->query_database($query, $query2);	
	}
	public function update_subscriber($email, $first_name="", $last_name="", $custom="", $customtwo="") {
		$email 		= $this->cleanup_injections($email);
		$first_name = $this->cleanup_injections($first_name);
		$last_name  = $this->cleanup_injections($last_name);
        $custom     = $this->cleanup_injections($custom);
        $customtwo  = $this->cleanup_injections($customtwo);

		$query = "UPDATE active_mailing SET first_name = $first_name WHERE email = $email";
		return $this->query_database($query);
	}
	public function remove_subscriber($email, $first_name="", $last_name="", $custom="", $customtwo="") {
		$email 		= $this->cleanup_injections($email);
		$first_name = $this->cleanup_injections($first_name);
		$last_name  = $this->cleanup_injections($last_name);
		$custom     = $this->cleanup_injections($custom);
		$customtwo  = $this->cleanup_injections($customtwo);

		$query = "DELETE FROM active_mailing WHERE email = $email";
		$query2 = "INSERT IGNORE INTO ignore_mailing (first_name, last_name, email, customone, customtwo) VALUES($first_name, $last_name, $email, $custom, $customtwo)";
		return $this->query_database($query, $query2);	
	}
	public function export_subscribers($auth) {
		// Make sure the the provided auth code matches
		if ($auth === $this->export_auth) {
			$result = mysql_query("SELECT * FROM active_mailing", $this->db_conn);
		
			$csv_file = $this->data_dir.'/postoffice-'.md5(uniqid(time())).'.csv';
			$fh = fopen($csv_file, 'w');
			$header = "First Name,Last Name,Email,Custom Field 1,Custom Field 2\n";
			fwrite($fh, $header);
		
			while($row = mysql_fetch_array($result)) {
				$str = $row['first_name'].",".$row['last_name'].",".$row['email'].",".$row['customone'].",".$row['customtwo']."\n";
				fwrite($fh, $str);
		  	}
			fclose($fh);
				
			echo '<script>document.location.href = "'.$csv_file.'"</script>';
		}
		return;
	}
   
} // End PostOfficeAPI Class
?>
