<?php
/* Post Office Stack Subscription Manager for Sendy
 * © Joe Workman 2015 */
class SendyAPI {

	private $log_to_console;

	public function __construct($server, $list, $log_to_console = true) {
		$this->url = $server;
		$this->list = $list;
		$this->log_to_console = $log_to_console;
	}

	//---------------------------------------
	// HTTP POST Functions
	//---------------------------------------
	private function httpPost($url,$params) {
		//open connection
		$ch = curl_init();

		//set the url, number of POST vars, POST data
		curl_setopt($ch,CURLOPT_URL,$url);
		curl_setopt($ch,CURLOPT_POST,count($params));
		curl_setopt($ch,CURLOPT_POSTFIELDS,http_build_query($params));
		curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);

		//execute post
		$result = curl_exec($ch);

		//close connection
		curl_close($ch);

		return $result;
	}
	private function parseResult($result) {
		$result = trim($result);
		if ($result === '1' || $result === 'Already subscribed.') return true;
		return false;
	}

	//---------------------------------------
	// Error Logging
	//---------------------------------------
	// set_error_handler("custom_error");
	private function custom_error($errno, $string) {
		return $this->log_message("Error Handler: [$errno] $string");
	}
	private function log_message($string) {
		if ($this->log_to_console) {
			echo '<script type="text/javascript">';
			echo "console.log(\"Post Office: $string\");";
			echo '</script>';
		}
		return;
	}

	//---------------------------------------
	// Public Subscribe Functions
	//---------------------------------------
	public function add_subscriber($email, $name) {
		$email = trim($email);
		$name  = trim($name);

		//set POST variables
		$url = $this->url.'/subscribe';
		$fields = array(
			'name'    => urlencode($name),
			'email'   => $email,
			'list'    => urlencode($this->list),
			'boolean' => 'true'
		);
		$result = $this->httpPost($url,$fields);

		if ($this->parseResult($result)) return true;

		$this->log_message("Problem Subscribing: $result");
		return false;
	}
	public function remove_subscriber($email) {
		$email = trim($email);

		//set POST variables
		$url = $this->url.'/unsubscribe';
		$fields = array(
			'email'   => $email,
			'list'    => urlencode($this->list),
			'boolean' => 'true'
		);
		$result = $this->httpPost($url,$fields);

		if ($this->parseResult($result)) return true;

		$this->log_message("Problem Unsubscribing: $result");
		return false;
	}
} // End SendyAPI Class
?>
